package cn.ibizlab.core.sample.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import cn.ibizlab.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.core.sample.domain.IBIZCustomerMG;
import cn.ibizlab.core.sample.filter.IBIZCustomerMGSearchContext;
import cn.ibizlab.core.sample.service.IIBIZCustomerMGService;

import cn.ibizlab.util.helper.CachedBeanCopier;
import cn.ibizlab.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.core.sample.mapper.IBIZCustomerMGMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[客户管理] 服务对象接口实现
 */
@Slf4j
@Service("IBIZCustomerMGServiceImpl")
public class IBIZCustomerMGServiceImpl extends ServiceImpl<IBIZCustomerMGMapper, IBIZCustomerMG> implements IIBIZCustomerMGService {


    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(IBIZCustomerMG et) {
        if(!this.retBool(this.baseMapper.insert(et))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getIbizcustomerid()), et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<IBIZCustomerMG> list) {
        this.saveBatch(list, batchSize);
    }

    @Override
    @Transactional
    public boolean update(IBIZCustomerMG et) {
        if(!update(et, (Wrapper) et.getUpdateWrapper(true).eq("ibizcustomerid", et.getIbizcustomerid()))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getIbizcustomerid()), et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<IBIZCustomerMG> list) {
        updateBatchById(list, batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result = removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public IBIZCustomerMG get(String key) {
        IBIZCustomerMG et = getById(key);
        if(et == null){
            et = new IBIZCustomerMG();
            et.setIbizcustomerid(key);
        }
        else {
        }
        return et;
    }

    @Override
    public IBIZCustomerMG getDraft(IBIZCustomerMG et) {
        return et;
    }

    @Override
    public boolean checkKey(IBIZCustomerMG et) {
        return (!ObjectUtils.isEmpty(et.getIbizcustomerid())) && (!Objects.isNull(this.getById(et.getIbizcustomerid())));
    }
    @Override
    @Transactional
    public boolean save(IBIZCustomerMG et) {
        if(!saveOrUpdate(et)) {
            return false;
        }
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(IBIZCustomerMG et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    @Transactional
    public boolean saveBatch(Collection<IBIZCustomerMG> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    @Transactional
    public void saveBatch(List<IBIZCustomerMG> list) {
        saveOrUpdateBatch(list,batchSize);
    }



    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<IBIZCustomerMG> searchDefault(IBIZCustomerMGSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBIZCustomerMG> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBIZCustomerMG>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }







    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }





}



