package cn.ibizlab.core.sample.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import cn.ibizlab.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.core.sample.domain.IBIZCustomer;
import cn.ibizlab.core.sample.filter.IBIZCustomerSearchContext;
import cn.ibizlab.core.sample.service.IIBIZCustomerService;

import cn.ibizlab.util.helper.CachedBeanCopier;
import cn.ibizlab.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.core.sample.mapper.IBIZCustomerMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[客户] 服务对象接口实现
 */
@Slf4j
@Service("IBIZCustomerServiceImpl")
public class IBIZCustomerServiceImpl extends ServiceImpl<IBIZCustomerMapper, IBIZCustomer> implements IIBIZCustomerService {

    @Autowired
    @Lazy
    protected cn.ibizlab.core.sample.service.IIBIZOrderService ibizorderService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(IBIZCustomer et) {
        if(!this.retBool(this.baseMapper.insert(et))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getIbizcustomerid()), et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<IBIZCustomer> list) {
        this.saveBatch(list, batchSize);
    }

    @Override
    @Transactional
    public boolean update(IBIZCustomer et) {
        if(!update(et, (Wrapper) et.getUpdateWrapper(true).eq("ibizcustomerid", et.getIbizcustomerid()))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getIbizcustomerid()), et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<IBIZCustomer> list) {
        updateBatchById(list, batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result = removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public IBIZCustomer get(String key) {
        IBIZCustomer et = getById(key);
        if(et == null){
            et = new IBIZCustomer();
            et.setIbizcustomerid(key);
        }
        else {
        }
        return et;
    }

    @Override
    public IBIZCustomer getDraft(IBIZCustomer et) {
        return et;
    }

    @Override
    public boolean checkKey(IBIZCustomer et) {
        return (!ObjectUtils.isEmpty(et.getIbizcustomerid())) && (!Objects.isNull(this.getById(et.getIbizcustomerid())));
    }
    @Override
    @Transactional
    public boolean save(IBIZCustomer et) {
        if(!saveOrUpdate(et)) {
            return false;
        }
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(IBIZCustomer et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    @Transactional
    public boolean saveBatch(Collection<IBIZCustomer> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    @Transactional
    public void saveBatch(List<IBIZCustomer> list) {
        saveOrUpdateBatch(list,batchSize);
    }



    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<IBIZCustomer> searchDefault(IBIZCustomerSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBIZCustomer> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBIZCustomer>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }







    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<IBIZCustomer> getIbizcustomerByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<IBIZCustomer> getIbizcustomerByEntities(List<IBIZCustomer> entities) {
        List ids =new ArrayList();
        for(IBIZCustomer entity : entities){
            Serializable id=entity.getIbizcustomerid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0) {
            return this.listByIds(ids);
        }
        else {
            return entities;
        }
    }




}



