package cn.ibizlab.core.sample.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import cn.ibizlab.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.core.sample.domain.IBIZHardware;
import cn.ibizlab.core.sample.filter.IBIZHardwareSearchContext;
import cn.ibizlab.core.sample.service.IIBIZHardwareService;

import cn.ibizlab.util.helper.CachedBeanCopier;
import cn.ibizlab.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.core.sample.mapper.IBIZHardwareMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[硬件] 服务对象接口实现
 */
@Slf4j
@Service("IBIZHardwareServiceImpl")
public class IBIZHardwareServiceImpl extends ServiceImpl<IBIZHardwareMapper, IBIZHardware> implements IIBIZHardwareService {


    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(IBIZHardware et) {
        createIndexMajorEntityData(et);
        if(!this.retBool(this.baseMapper.insert(et))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getIbizhardwareid()), et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<IBIZHardware> list) {
        this.saveBatch(list, batchSize);
    }

    @Override
    @Transactional
    public boolean update(IBIZHardware et) {
        ibizuniproductService.update(ibizhardwareInheritMapping.toIbizuniproduct(et));
        if(!update(et, (Wrapper) et.getUpdateWrapper(true).eq("ibizhardwareid", et.getIbizhardwareid()))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getIbizhardwareid()), et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<IBIZHardware> list) {
        updateBatchById(list, batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        ibizuniproductService.remove(key);
        boolean result = removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public IBIZHardware get(String key) {
        IBIZHardware et = getById(key);
        if(et == null){
            et = new IBIZHardware();
            et.setIbizhardwareid(key);
        }
        else {
        }
        return et;
    }

    @Override
    public IBIZHardware getDraft(IBIZHardware et) {
        return et;
    }

    @Override
    public boolean checkKey(IBIZHardware et) {
        return (!ObjectUtils.isEmpty(et.getIbizhardwareid())) && (!Objects.isNull(this.getById(et.getIbizhardwareid())));
    }
    @Override
    @Transactional
    public boolean save(IBIZHardware et) {
        if(!saveOrUpdate(et)) {
            return false;
        }
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(IBIZHardware et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    @Transactional
    public boolean saveBatch(Collection<IBIZHardware> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    @Transactional
    public void saveBatch(List<IBIZHardware> list) {
        saveOrUpdateBatch(list,batchSize);
    }



    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<IBIZHardware> searchDefault(IBIZHardwareSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBIZHardware> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBIZHardware>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 IndexDER
     */
    @Override
    public Page<IBIZHardware> searchIndexDER(IBIZHardwareSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBIZHardware> pages=baseMapper.searchIndexDER(context.getPages(),context,context.getSelectCond());
        return new PageImpl<IBIZHardware>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }






    @Autowired
    cn.ibizlab.core.sample.mapping.IBIZHardwareInheritMapping ibizhardwareInheritMapping;
    @Autowired
    @Lazy
    protected cn.ibizlab.core.sample.service.IIBIZUNIProductService ibizuniproductService;

    /**
     * 创建索引主实体数据
     * @param et
     */
    private void createIndexMajorEntityData(IBIZHardware et){
        if(ObjectUtils.isEmpty(et.getIbizhardwareid()))
            et.setIbizhardwareid((String)et.getDefaultKey(true));
        cn.ibizlab.core.sample.domain.IBIZUNIProduct ibizuniproduct =ibizhardwareInheritMapping.toIbizuniproduct(et);
        ibizuniproduct.set("ibizuniproducttype","Hardware");
        ibizuniproductService.create(ibizuniproduct);
    }

    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<IBIZHardware> getIbizhardwareByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<IBIZHardware> getIbizhardwareByEntities(List<IBIZHardware> entities) {
        List ids =new ArrayList();
        for(IBIZHardware entity : entities){
            Serializable id=entity.getIbizhardwareid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0) {
            return this.listByIds(ids);
        }
        else {
            return entities;
        }
    }




}



